@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Generic "memory" (stored bitmap) device */
@@ -133,6 +133,9 @@ gs_device_is_memory(const gx_device * dev)
     int bits_per_pixel = dev->color_info.depth;
     const gx_device_memory *mdproto;
 
+    if (dev->is_planar)
+        bits_per_pixel /= dev->color_info.num_components;
+
     mdproto = gdev_mem_device_for_bits(bits_per_pixel);
     if (mdproto != 0 && dev_proc(dev, draw_thin_line) == dev_proc(mdproto, draw_thin_line))
         return true;
@@ -173,12 +176,12 @@ gs_make_mem_device(gx_device_memory * dev, const gx_device_memory * mdproto,
         /* Forward the color mapping operations to the target. */
         gx_device_forward_color_procs((gx_device_forward *) dev);
         gx_device_copy_color_procs((gx_device *)dev, target);
+        dev->color_info.separable_and_linear = target->color_info.separable_and_linear;
         dev->cached_colors = target->cached_colors;
         dev->graphics_type_tag = target->graphics_type_tag;	/* initialize to same as target */
         /* Do a copy of put_image since it needs the source buffer */
-#define COPY_PROC(p) set_dev_proc(dev, p, target->procs.p)
-        COPY_PROC(put_image);
-#undef COPY_PROC
+        set_dev_proc(dev, put_image, target->procs.put_image);
+        set_dev_proc(dev, dev_spec_op, gx_default_dev_spec_op);
     }
     if (dev->color_info.depth == 1) {
         gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
@@ -305,6 +308,7 @@ gs_make_mem_mono_device(gx_device_memory * dev, gs_memory_t * mem,
     /* Should this be forwarding, monochrome profile, or not set? MJV */
     set_dev_proc(dev, get_profile, gx_forward_get_profile);
     set_dev_proc(dev, set_graphics_type_tag, gx_forward_set_graphics_type_tag);
+    set_dev_proc(dev, dev_spec_op, gx_default_dev_spec_op);
     /* initialize to same tag as target */
     dev->graphics_type_tag = target ? target->graphics_type_tag : GS_UNKNOWN_TAG;
 }
@@ -605,6 +609,8 @@ mem_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
             GB_PACKING_CHUNKY | GB_COLORS_NATIVE | GB_ALPHA_NONE;
         return_error(gs_error_rangecheck);
     }
+    if (mdev->line_ptrs == 0x00)
+        return_error(gs_error_rangecheck);
     if ((w <= 0) | (h <= 0)) {
         if ((w | h) < 0)
             return_error(gs_error_rangecheck);
@@ -707,6 +713,10 @@ mem_word_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
     }
     bit_x = x * dev->color_info.depth;
     bit_w = w * dev->color_info.depth;
+
+    if(mdev->line_ptrs == 0x00)
+        return_error(gs_error_rangecheck);
+
     src = scan_line_base(mdev, y);
     mem_swap_byte_rect(src, dev_raster, bit_x, bit_w, h, false);
     code = mem_get_bits_rectangle(dev, prect, params, unread);
