@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Any-depth planar "memory" (stored bitmap) device */
@@ -51,7 +51,7 @@ mem_planar_dev_spec_op(gx_device *pdev, int dev_spec_op,
     if (dev_spec_op == gxdso_supports_devn) {
         dev_proc(pdev, get_profile)(pdev, &dev_profile);
         if (dev_profile != NULL && dev_profile->supports_devn &&
-            pdev->procs.fill_rectangle_hl_color == mem_planar_fill_rectangle_hl_color)
+            dev_proc(pdev, fill_rectangle_hl_color) == mem_planar_fill_rectangle_hl_color)
             return 1;
     }
     return gx_default_dev_spec_op(pdev, dev_spec_op, data, size);
@@ -115,6 +115,12 @@ gdev_mem_set_planar(gx_device_memory * mdev, int num_planes,
     mdev->plane_depth = same_depth;
     /* Change the drawing procedures. */
     set_dev_proc(mdev, open_device, mem_planar_open);
+    /* Regardless of how many planes we are using, always let the
+     * device know how to handle hl_color. Even if we spot that we
+     * can get away with a normal device, our callers may want to
+     * feed us single component devn data. */
+    set_dev_proc(mdev, fill_rectangle_hl_color,
+                 mem_planar_fill_rectangle_hl_color);
     if (num_planes == 1) {
         /* For 1 plane, just use a normal device */
         set_dev_proc(mdev, fill_rectangle, dev_proc(mdproto, fill_rectangle));
@@ -131,8 +137,6 @@ gdev_mem_set_planar(gx_device_memory * mdev, int num_planes,
            make use of the put_image operation to go from the pdf14 device
            directly to the planar buffer. */
         if (num_planes >= 4) {
-            set_dev_proc(mdev, fill_rectangle_hl_color,
-                         mem_planar_fill_rectangle_hl_color);
             set_dev_proc(mdev, put_image, mem_planar_put_image);
         }
         set_dev_proc(mdev, fill_rectangle, mem_planar_fill_rectangle);
@@ -850,6 +854,14 @@ mem_planar_copy_color(gx_device * dev, const byte * base, int sourcex,
     mem_save_params_t save;
     uchar pi;
 
+    /* This routine cannot copy from 3bit chunky data, as 3 bit
+     * things don't pack nicely into bytes or words. Accordingly
+     * treat 3 bit things as 4 bit things. This is appropriate as
+     * 3 bit data will generally have been passed to us as 4bit
+     * data - such as halftones. */
+    if (source_depth == 3)
+        source_depth = 4;
+
     fit_copy(dev, base, sourcex, sraster, id, x, y, w, h);
     MEM_SAVE_PARAMS(mdev, save);
     for (pi = 0; pi < mdev->color_info.num_components; ++pi) {
@@ -1092,7 +1104,7 @@ planar_cmyk4bit_strip_copy_rop(gx_device_memory * mdev,
     int line_count;
     byte *cdrow, *mdrow, *ydrow, *kdrow;
     byte lmask, rmask;
-    rop_proc cproc, mproc, yproc;
+    rop_proc cproc = NULL, mproc = NULL, yproc = NULL;
     int dbit;
     int cscolor = 0, mscolor = 0, yscolor = 0, kscolor = 0;
     int ctcolor = 0, mtcolor = 0, ytcolor = 0, ktcolor = 0;
@@ -2085,6 +2097,10 @@ mem_planar_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
             GB_COLORS_NATIVE | GB_ALPHA_NONE;
         return_error(gs_error_rangecheck);
     }
+
+    if (mdev->line_ptrs == 0x00)
+        return_error(gs_error_rangecheck);
+
     if ((w <= 0) | (h <= 0)) {
         if ((w | h) < 0)
             return_error(gs_error_rangecheck);
