@@ -220,6 +220,7 @@ gdev_prn_setup_as_command_list(gx_device *pdev, gs_memory_t *buffer_memory,
                                bool bufferSpace_is_exact)
 {
     gx_device_printer * const ppdev = (gx_device_printer *)pdev;
+    gx_device *target = pdev;
     uint space;
     int code;
     gx_device_clist *const pclist_dev = (gx_device_clist *)pdev;
@@ -227,6 +228,11 @@ gdev_prn_setup_as_command_list(gx_device *pdev, gs_memory_t *buffer_memory,
     bool reallocate = *the_memory != 0;
     byte *base;
 
+    while (target->parent != NULL) {
+        target = target->parent;
+        gx_update_from_subclass(target);
+    }
+
     /* Try to allocate based simply on param-requested buffer size */
 #ifdef DEBUGGING_HACKS
 #define BACKTRACE(first_arg)\
@@ -260,7 +266,7 @@ open_c:
     ppdev->buffer_space = space;
     pclist_dev->common.is_printer = 1;
     clist_init_io_procs(pclist_dev, ppdev->BLS_force_memory);
-    clist_init_params(pclist_dev, base, space, pdev,
+    clist_init_params(pclist_dev, base, space, target,
                       ppdev->printer_procs.buf_procs,
                       space_params->band,
                       false, /* do_not_open_or_close_bandfiles */
@@ -448,11 +454,14 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 
             /* Try to allocate memory for full memory buffer, then allocate the
                pdf14_trans_buffer_size to make sure we have enough space for that */
-            base = (reallocate ?
-                       (byte *)gs_resize_object(buffer_memory, the_memory,
-                                         (uint)mem_space, "printer buffer") :
-                       gs_alloc_bytes(buffer_memory, (uint)mem_space,
-                                      "printer_buffer"));
+            /* NOTE: Assumes that caller won't normally call this function if page
+               size didn't actually change, so we can free/alloc without checking
+               that the new size is different than old size.
+            */
+            if (reallocate) {
+                gs_free_object(buffer_memory, the_memory, "printer_buffer");
+            }
+            base = gs_alloc_bytes(buffer_memory, (uint)mem_space, "printer_buffer");
             if (base == 0)
                 is_command_list = true;
             else
