@@ -347,7 +347,7 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
         /* Fastest case.  No decode or CM needed */
         *psrc_cm = (unsigned char *) psrc;
         spp_cm = spp;
-        *bufend = *psrc_cm +  w;
+        *bufend = *psrc_cm + w;
         *psrc_cm_start = NULL;
     } else {
         spp_cm = num_des_comps;
@@ -878,119 +878,69 @@ image_render_color_icc_portrait(gx_image_enum *penum_orig, const byte *buffer, i
     const gs_gstate *pgs = penum->pgs;
     gs_logical_operation_t lop = penum->log_op;
     gx_dda_fixed_point pnext;
-    fixed xprev;
     int vci, vdi;
-    gx_device_color devc1;
-    gx_device_color devc2;
-    gx_device_color *pdevc;
-    gx_device_color *pdevc_next;
-    gx_device_color *ptemp;
     int spp = penum->spp;
-    const byte *psrc_initial = buffer + data_x * spp;
-    const byte *psrc = psrc_initial;
-    const byte *rsrc = psrc + spp; /* psrc + spp at start of run */
+    const byte *psrc = buffer + data_x * spp;
     int irun;			/* int x/rrun */
     byte *bufend = NULL;
     int code = 0;
     byte *psrc_cm = NULL, *psrc_cm_start = NULL;
+    byte *psrc_cm_initial;
+    byte *run;
     int k;
-    gx_color_value conc[GX_DEVICE_COLOR_MAX_COMPONENTS];
     int spp_cm = 0;
-    gx_color_index color;
     bool must_halftone = penum->icc_setup.must_halftone;
     bool has_transfer = penum->icc_setup.has_transfer;
-    byte initial_run[GX_DEVICE_COLOR_MAX_COMPONENTS] = { 0 };
-    byte *run = &initial_run[0]; /* run value */
+    gx_cmapper_data data;
+    gx_cmapper_fn *mapper = gx_get_cmapper(&data, pgs, dev, has_transfer, must_halftone, gs_color_select_source);
+    gx_color_value *conc = &data.conc[0];
 
-    pdevc = &devc1;
-    pdevc_next = &devc2;
-    /* These used to be set by init clues */
-    pdevc->type = gx_dc_type_none;
-    pdevc_next->type = gx_dc_type_none;
     if (h == 0)
         return 0;
     code = image_color_icc_prep(penum_orig, psrc, w, dev, &spp_cm, &psrc_cm,
                                 &psrc_cm_start, &bufend, false);
     if (code < 0) return code;
+    psrc_cm_initial = psrc_cm;
     /* Needed for device N */
-    memset(&(conc[0]), 0, sizeof(gx_color_value[GX_DEVICE_COLOR_MAX_COMPONENTS]));
     pnext = penum->dda.pixel0;
-    xprev = dda_current(pnext.x);
     dda_translate(pnext.x,  (-fixed_epsilon));
+    irun = fixed2int_var_rounded(dda_current(pnext.x));
     vci = penum->yci, vdi = penum->hci;
-    irun = fixed2int_var_rounded(xprev);
     if_debug5m('b', penum->memory, "[b]y=%d data_x=%d w=%d xt=%f yt=%f\n",
-               penum->y, data_x, w, fixed2float(xprev), fixed2float(dda_current(pnext.y)));
-    run[0] = ~psrc_cm[0];	/* Force intial setting */
+               penum->y, data_x, w, fixed2float(dda_current(pnext.x)), fixed2float(dda_current(pnext.y)));
     while (psrc_cm < bufend) {
-        dda_next(pnext.x);
-
-        if (memcmp(run, psrc_cm, spp_cm) == 0)
-            goto inc;
+        /* Find the length of the next run. It will either end when we hit
+         * the end of the source data, or when the pixel data differs. */
+        run = psrc_cm + spp_cm;
+        while (1)
+        {
+            dda_next(pnext.x);
+            if (run >= bufend)
+                break;
+            if (memcmp(run, psrc_cm, spp_cm))
+                break;
+            run += spp_cm;
+        }
+        /* So we have a run of pixels from psrc_cm to run that are all the same. */
         /* This needs to be sped up */
         for (k = 0; k < spp_cm; k++) {
             conc[k] = gx_color_value_from_byte(psrc_cm[k]);
         }
-        /* Now we can do an encoding directly or we have to apply transfer
-           and or halftoning */
-        if (must_halftone || has_transfer) {
-            /* We need to do the tranfer function and/or the halftoning */
-            cmap_transfer_halftone(&(conc[0]), pdevc_next, pgs, dev,
-                has_transfer, must_halftone, gs_color_select_source);
-        } else {
-            /* encode as a color index. avoid all the cv to frac to cv
-               conversions */
-            color = dev_proc(dev, encode_color)(dev, &(conc[0]));
-            /* check if the encoding was successful; we presume failure is rare */
-            if (color != gx_no_color_index)
-                color_set_pure(pdevc_next, color);
-        }
-        /* Fill the region between */
-        /* irun and xprev */
-        /*
-         * Note;  This section is nearly a copy of a simlar section below
-         * for processing the last image pixel in the loop.  This would have been
-         * made into a subroutine except for complications about the number of
-         * variables that would have been needed to be passed to the routine.
-         */
-        {		/* Rectangle */
+        mapper(&data);
+        /* Fill the region between irun and fixed2int_var_rounded(pnext.x) */
+        {
             int xi = irun;
-            int wi = (irun = fixed2int_var_rounded(xprev)) - xi;
+            int wi = (irun = fixed2int_var_rounded(dda_current(pnext.x))) - xi;
 
             if (wi < 0)
                 xi += wi, wi = -wi;
             if (wi > 0)
                 code = gx_fill_rectangle_device_rop(xi, vci, wi, vdi,
-                                                    pdevc, dev, lop);
+                                                    &data.devc, dev, lop);
         }
         if (code < 0)
             goto err;
-        rsrc = psrc;
-        /* Swap around the colors due to a change */
-        ptemp = pdevc;
-        pdevc = pdevc_next;
-        pdevc_next = ptemp;
-        run = psrc_cm;
-inc:
-        psrc_cm += spp_cm;
-        xprev = dda_current(pnext.x);
-    }
-    /* Fill the last run. */
-    /*
-     * Note;  This section is nearly a copy of a simlar section above
-     * for processing an image pixel in the loop.  This would have been
-     * made into a subroutine except for complications about the number
-     * variables that would have been needed to be passed to the routine.
-     */
-    {		/* Rectangle */
-        int xi = irun;
-        int wi = (irun = fixed2int_var_rounded(xprev)) - xi;
-
-        if (wi < 0)
-            xi += wi, wi = -wi;
-        if (wi > 0)
-            code = gx_fill_rectangle_device_rop(xi, vci, wi, vdi,
-                                                pdevc, dev, lop);
+        psrc_cm = run;
     }
     /* Free cm buffer, if it was used */
     if (psrc_cm_start != NULL) {
@@ -1000,7 +950,7 @@ inc:
     /* Save position if error, in case we resume. */
 err:
     gs_free_object(pgs->memory, (byte *)psrc_cm_start, "image_render_color_icc");
-    penum_orig->used.x = (rsrc - spp - psrc_initial) / spp;
+    penum_orig->used.x = (run - psrc_cm_initial) / spp_cm;
     penum_orig->used.y = 0;
     return code;
 }
@@ -1013,119 +963,69 @@ image_render_color_icc_landscape(gx_image_enum *penum_orig, const byte *buffer,
     const gs_gstate *pgs = penum->pgs;
     gs_logical_operation_t lop = penum->log_op;
     gx_dda_fixed_point pnext;
-    fixed yprev;
     int vci, vdi;
-    gx_device_color devc1;
-    gx_device_color devc2;
-    gx_device_color *pdevc;
-    gx_device_color *pdevc_next;
-    gx_device_color *ptemp;
     int spp = penum->spp;
-    const byte *psrc_initial = buffer + data_x * spp;
-    const byte *psrc = psrc_initial;
-    const byte *rsrc = psrc + spp; /* psrc + spp at start of run */
+    const byte *psrc = buffer + data_x * spp;
     int irun;			/* int x/rrun */
     byte *bufend = NULL;
     int code = 0;
     byte *psrc_cm = NULL, *psrc_cm_start = NULL;
+    byte *psrc_cm_initial;
+    byte *run;
     int k;
-    gx_color_value conc[GX_DEVICE_COLOR_MAX_COMPONENTS];
     int spp_cm = 0;
-    gx_color_index color;
     bool must_halftone = penum->icc_setup.must_halftone;
     bool has_transfer = penum->icc_setup.has_transfer;
-    byte initial_run[GX_DEVICE_COLOR_MAX_COMPONENTS] = { 0 };
-    byte *run = &initial_run[0]; /* run value */
+    gx_cmapper_data data;
+    gx_cmapper_fn *mapper = gx_get_cmapper(&data, pgs, dev, has_transfer, must_halftone, gs_color_select_source);
+    gx_color_value *conc = &data.conc[0];
 
-    pdevc = &devc1;
-    pdevc_next = &devc2;
-    /* These used to be set by init clues */
-    pdevc->type = gx_dc_type_none;
-    pdevc_next->type = gx_dc_type_none;
     if (h == 0)
         return 0;
     code = image_color_icc_prep(penum_orig, psrc, w, dev, &spp_cm, &psrc_cm,
                                 &psrc_cm_start, &bufend, false);
     if (code < 0) return code;
+    psrc_cm_initial = psrc_cm;
     /* Needed for device N */
-    memset(&(conc[0]), 0, sizeof(gx_color_value[GX_DEVICE_COLOR_MAX_COMPONENTS]));
     pnext = penum->dda.pixel0;
     dda_translate(pnext.x,  (-fixed_epsilon));
-    yprev = dda_current(pnext.y);
+    irun = fixed2int_var_rounded(dda_current(pnext.y));
     vci = penum->xci, vdi = penum->wci;
-    irun = fixed2int_var_rounded(yprev);
     if_debug5m('b', penum->memory, "[b]y=%d data_x=%d w=%d xt=%f yt=%f\n",
-               penum->y, data_x, w, fixed2float(dda_current(pnext.x)), fixed2float(yprev));
-    run[0] = ~psrc_cm[0];	/* Force intial setting */
+               penum->y, data_x, w, fixed2float(dda_current(pnext.x)), fixed2float(dda_current(pnext.y)));
     while (psrc_cm < bufend) {
-        dda_next(pnext.y);
-
-        if (memcmp(run, psrc_cm, spp_cm) == 0)
-            goto inc;
+        /* Find the length of the next run. It will either end when we hit
+         * the end of the source data, or when the pixel data differs. */
+        run = psrc_cm + spp_cm;
+        while (1)
+        {
+            dda_next(pnext.y);
+            if (run >= bufend)
+                break;
+            if (memcmp(run, psrc_cm, spp_cm))
+                break;
+            run += spp_cm;
+        }
+        /* So we have a run of pixels from psrc_cm to run that are all the same. */
         /* This needs to be sped up */
         for (k = 0; k < spp_cm; k++) {
             conc[k] = gx_color_value_from_byte(psrc_cm[k]);
         }
-        /* Now we can do an encoding directly or we have to apply transfer
-           and or halftoning */
-        if (must_halftone || has_transfer) {
-            /* We need to do the tranfer function and/or the halftoning */
-            cmap_transfer_halftone(&(conc[0]), pdevc_next, pgs, dev,
-                has_transfer, must_halftone, gs_color_select_source);
-        } else {
-            /* encode as a color index. avoid all the cv to frac to cv
-               conversions */
-            color = dev_proc(dev, encode_color)(dev, &(conc[0]));
-            /* check if the encoding was successful; we presume failure is rare */
-            if (color != gx_no_color_index)
-                color_set_pure(pdevc_next, color);
-        }
-        /* Fill the region between */
-        /* irun and yprev */
-        /*
-         * Note;  This section is nearly a copy of a simlar section below
-         * for processing the last image pixel in the loop.  This would have been
-         * made into a subroutine except for complications about the number of
-         * variables that would have been needed to be passed to the routine.
-         */
+        mapper(&data);
+        /* Fill the region between irun and fixed2int_var_rounded(pnext.y) */
         {		/* 90 degree rotated rectangle */
             int yi = irun;
-            int hi = (irun = fixed2int_var_rounded(yprev)) - yi;
+            int hi = (irun = fixed2int_var_rounded(dda_current(pnext.y))) - yi;
 
             if (hi < 0)
                 yi += hi, hi = -hi;
             if (hi > 0)
                 code = gx_fill_rectangle_device_rop(vci, yi, vdi, hi,
-                                                    pdevc, dev, lop);
+                                                    &data.devc, dev, lop);
         }
         if (code < 0)
             goto err;
-        rsrc = psrc;
-        /* Swap around the colors due to a change */
-        ptemp = pdevc;
-        pdevc = pdevc_next;
-        pdevc_next = ptemp;
-        run = psrc_cm;
-inc:
-        psrc_cm += spp_cm;
-        yprev = dda_current(pnext.y);	/* harmless if no skew */
-    }
-    /* Fill the last run. */
-    /*
-     * Note;  This section is nearly a copy of a simlar section above
-     * for processing an image pixel in the loop.  This would have been
-     * made into a subroutine except for complications about the number
-     * variables that would have been needed to be passed to the routine.
-     */
-    {		/* 90 degree rotated rectangle */
-        int yi = irun;
-        int hi = (irun = fixed2int_var_rounded(yprev)) - yi;
-
-        if (hi < 0)
-            yi += hi, hi = -hi;
-        if (hi > 0)
-            code = gx_fill_rectangle_device_rop(vci, yi, vdi, hi,
-                                                pdevc, dev, lop);
+        psrc_cm = run;
     }
     /* Free cm buffer, if it was used */
     if (psrc_cm_start != NULL) {
@@ -1135,7 +1035,7 @@ inc:
     /* Save position if error, in case we resume. */
 err:
     gs_free_object(pgs->memory, (byte *)psrc_cm_start, "image_render_color_icc");
-    penum_orig->used.x = (rsrc - spp - psrc_initial) / spp;
+    penum_orig->used.x = (run - psrc_cm_initial) / spp_cm;
     penum_orig->used.y = 0;
     return code;
 }
@@ -1150,108 +1050,67 @@ image_render_color_icc_skew(gx_image_enum *penum_orig, const byte *buffer, int d
     gx_dda_fixed_point pnext;
     fixed xprev, yprev;
     fixed pdyx, pdyy;		/* edge of parallelogram */
-    gx_device_color devc1;
-    gx_device_color devc2;
-    gx_device_color *pdevc;
-    gx_device_color *pdevc_next;
-    gx_device_color *ptemp;
     int spp = penum->spp;
-    const byte *psrc_initial = buffer + data_x * spp;
-    const byte *psrc = psrc_initial;
-    const byte *rsrc = psrc + spp; /* psrc + spp at start of run */
-    fixed xrun;			/* x ditto */
-    fixed yrun;			/* y ditto */
+    const byte *psrc = buffer + data_x * spp;
+    fixed xpos;			/* x ditto */
+    fixed ypos;			/* y ditto */
     byte *bufend = NULL;
     int code = 0;
     byte *psrc_cm = NULL, *psrc_cm_start = NULL;
+    byte *psrc_cm_initial;
     int k;
-    gx_color_value conc[GX_DEVICE_COLOR_MAX_COMPONENTS];
     int spp_cm = 0;
-    gx_color_index color;
     bool must_halftone = penum->icc_setup.must_halftone;
     bool has_transfer = penum->icc_setup.has_transfer;
     byte initial_run[GX_DEVICE_COLOR_MAX_COMPONENTS] = { 0 };
-    byte *run = &initial_run[0]; /* run value */
+    byte *prev_cm = &initial_run[0];
+    gx_cmapper_data data;
+    gx_cmapper_fn *mapper = gx_get_cmapper(&data, pgs, dev, has_transfer, must_halftone, gs_color_select_source);
+    gx_color_value *conc = &data.conc[0];
 
-    pdevc = &devc1;
-    pdevc_next = &devc2;
-    /* These used to be set by init clues */
-    pdevc->type = gx_dc_type_none;
-    pdevc_next->type = gx_dc_type_none;
     if (h == 0)
         return 0;
     code = image_color_icc_prep(penum_orig, psrc, w, dev, &spp_cm, &psrc_cm,
                                 &psrc_cm_start, &bufend, false);
     if (code < 0) return code;
+    psrc_cm_initial = psrc_cm;
     /* Needed for device N */
-    memset(&(conc[0]), 0, sizeof(gx_color_value[GX_DEVICE_COLOR_MAX_COMPONENTS]));
     pnext = penum->dda.pixel0;
-    xrun = xprev = dda_current(pnext.x);
     dda_translate(pnext.x,  (-fixed_epsilon));
-    yrun = yprev = dda_current(pnext.y);
+    xprev = dda_current(pnext.x);
+    yprev = dda_current(pnext.y);
     pdyx = dda_current(penum->dda.row.x) - penum->cur.x;
     pdyy = dda_current(penum->dda.row.y) - penum->cur.y;
     if_debug5m('b', penum->memory, "[b]y=%d data_x=%d w=%d xt=%f yt=%f\n",
                penum->y, data_x, w, fixed2float(xprev), fixed2float(yprev));
-    run[0] = ~psrc_cm[0];	/* Force intial setting */
+    prev_cm[0] = ~psrc_cm[0];	/* Force intial setting */
     while (psrc_cm < bufend) {
         dda_next(pnext.x);
         dda_next(pnext.y);
+        xpos = dda_current(pnext.x);
+        ypos = dda_current(pnext.y);
 
-        /* This needs to be sped up */
-        for (k = 0; k < spp_cm; k++) {
-            conc[k] = gx_color_value_from_byte(psrc_cm[k]);
-        }
-        /* Now we can do an encoding directly or we have to apply transfer
-           and or halftoning */
-        if (must_halftone || has_transfer) {
-            /* We need to do the tranfer function and/or the halftoning */
-            cmap_transfer_halftone(&(conc[0]), pdevc_next, pgs, dev,
-                has_transfer, must_halftone, gs_color_select_source);
-        } else {
-            /* encode as a color index. avoid all the cv to frac to cv
-               conversions */
-            color = dev_proc(dev, encode_color)(dev, &(conc[0]));
-            /* check if the encoding was successful; we presume failure is rare */
-            if (color != gx_no_color_index)
-                color_set_pure(pdevc_next, color);
+        if (memcmp(prev_cm, psrc_cm, spp_cm) != 0)
+        {
+            /* This needs to be sped up */
+            for (k = 0; k < spp_cm; k++) {
+                conc[k] = gx_color_value_from_byte(psrc_cm[k]);
+            }
+            mapper(&data);
         }
         /* Fill the region between */
-        /* xrun/yrun and xprev/yprev */
-        /*
-         * Note;  This section is nearly a copy of a simlar section below
-         * for processing the last image pixel in the loop.  This would have been
-         * made into a subroutine except for complications about the number of
-         * variables that would have been needed to be passed to the routine.
-         */
+        /* xprev/yprev and xpos/ypos */
         /* Parallelogram */
         code = (*dev_proc(dev, fill_parallelogram))
-                    (dev, xrun, yrun, xprev - xrun, yprev - yrun, pdyx, pdyy,
-                     pdevc, lop);
-        xrun = xprev;
-        yrun = yprev;
+                    (dev, xprev, yprev, xpos - xprev, ypos - yprev, pdyx, pdyy,
+                     &data.devc, lop);
+        xprev = xpos;
+        yprev = ypos;
         if (code < 0)
             goto err;
-        rsrc = psrc;
-        /* Swap around the colors due to a change */
-        ptemp = pdevc;
-        pdevc = pdevc_next;
-        pdevc_next = ptemp;
+        prev_cm = psrc_cm;
         psrc_cm += spp_cm;
-        xprev = dda_current(pnext.x);
-        yprev = dda_current(pnext.y);	/* harmless if no skew */
     }
-    /* Fill the last run. */
-    /*
-     * Note;  This section is nearly a copy of a simlar section above
-     * for processing an image pixel in the loop.  This would have been
-     * made into a subroutine except for complications about the number
-     * variables that would have been needed to be passed to the routine.
-     */
-    /* Parallelogram */
-    code = (*dev_proc(dev, fill_parallelogram))
-                    (dev, xrun, yrun, xprev - xrun, yprev - yrun, pdyx, pdyy,
-                     pdevc, lop);
     /* Free cm buffer, if it was used */
     if (psrc_cm_start != NULL) {
         gs_free_object(pgs->memory, (byte *)psrc_cm_start, "image_render_color_icc");
@@ -1260,7 +1119,7 @@ image_render_color_icc_skew(gx_image_enum *penum_orig, const byte *buffer, int d
     /* Save position if error, in case we resume. */
 err:
     gs_free_object(pgs->memory, (byte *)psrc_cm_start, "image_render_color_icc");
-    penum_orig->used.x = (rsrc - spp - psrc_initial) / spp;
+    penum_orig->used.x = (psrc_cm - psrc_cm_initial) / spp_cm;
     penum_orig->used.y = 0;
     return code;
 }
