@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -233,17 +233,17 @@ const gx_device_bit gs_bitrgbtags_device =
         sizeof(gx_device_bit),
         &bitrgbtags_procs,
         "bitrgbtags",
-        0 ,                             /* memory */
+        0,                              /* memory */
         &st_device_printer,
-        0 ,                             /* stype_is_dynamic */
-        0 ,                             /* finalize */
-        { 0 } ,                         /* rc header */
-        0 ,                             /* retained */
-        0 ,                             /* parent */
-        0 ,                             /* child */
-        0 ,                             /* subclass data */
-        0 ,                             /* PageList */
-        0 ,                             /* is open */
+        0,                              /* stype_is_dynamic */
+        0,                              /* finalize */
+        { 0 },                          /* rc header */
+        0,                              /* retained */
+        0,                              /* parent */
+        0,                              /* child */
+        0,                              /* subclass data */
+        0,                              /* PageList */
+        0,                              /* is open */
         0,                              /* max_fill_band */
         {                               /* color infor */
             3,                          /* max_components */
@@ -251,17 +251,17 @@ const gx_device_bit gs_bitrgbtags_device =
             GX_CINFO_POLARITY_ADDITIVE, /* polarity */
             32,                         /* depth */
             GX_CINFO_COMP_NO_INDEX,     /* gray index */
-            255 ,                         /* max_gray */
-            255 ,                         /* max_colors */
-            256 ,                         /* dither grays */
-            256 ,                         /* dither colors */
-            { 1, 1 } ,                  /* antialiasing */
-            GX_CINFO_UNKNOWN_SEP_LIN,   /* sep and linear */
-            { 16, 8, 0, 0 } ,                    /* comp shift */
-            { 8, 8, 8, 8 } ,                     /* comp bits */
-            { 0xFF0000, 0x00FF00, 0x0000FF } ,     /* comp mask */
+            255,                        /* max_gray */
+            255,                        /* max_colors */
+            256,                        /* dither grays */
+            256,                        /* dither colors */
+            { 1, 1 },                   /* antialiasing */
+            GX_CINFO_SEP_LIN_STANDARD,  /* sep and linear */
+            { 16, 8, 0, 24 },           /* comp shift */	/* tag is upper byte */
+            { 8, 8, 8, 8 },             /* comp bits */
+            { 0xFF0000, 0x00FF00, 0x0000FF, 0xFF000000 },  /* comp mask */
             ( "DeviceRGBT" ),            /* color model name */
-            GX_CINFO_OPMODE_UNKNOWN ,   /* overprint mode */
+            GX_CINFO_OPMODE_UNKNOWN,     /* overprint mode */
             0,                           /* process comps */
             0                            /* icc_locations */
         },
@@ -269,86 +269,96 @@ const gx_device_bit gs_bitrgbtags_device =
             ((gx_color_index)(~0)),
             ((gx_color_index)(~0))
         },
-        (int)((float)(85) * (X_DPI) / 10 + 0.5),
-        (int)((float)(110) * (Y_DPI) / 10 + 0.5),
+        (int)((float)(85) * (X_DPI) / 10 + 0.5), /* width */
+        (int)((float)(110) * (Y_DPI) / 10 + 0.5),/* height */
         0, /* Pad */
-        0, /* Align */
-        0, /* Num planes */
-        0,
+        0, /* log2_align_mod */
+        0, /* is_planar */
+        0, /* LeadingEdge */
         {
-            (float)(((((int)((float)(85) * (X_DPI) / 10 + 0.5)) * 72.0 + 0.5) - 0.5) / (X_DPI)) ,
-            (float)(((((int)((float)(110) * (Y_DPI) / 10 + 0.5)) * 72.0 + 0.5) - 0.5) / (Y_DPI)) },
+            (float)(((((int)((float)(85) * (X_DPI) / 10 + 0.5)) * 72.0 + 0.5) - 0.5) / (X_DPI)),
+            (float)(((((int)((float)(110) * (Y_DPI) / 10 + 0.5)) * 72.0 + 0.5) - 0.5) / (Y_DPI))
+        }, /* MediaSize */
         {
             0,
             0,
             0,
             0
-        } ,
-        0 ,
-        { X_DPI, Y_DPI } ,
-        {(float)(-(0) * (X_DPI)),
-         (float)(-(0) * (Y_DPI))},
-        {(float)((0) * 72.0),
-         (float)((0) * 72.0),
-         (float)((0) * 72.0),
-         (float)((0) * 72.0)},
-        0 , /*FirstPage*/
-        0 , /*LastPage*/
-        0 , /*PageHandlerPushed*/
-        0 , /*DisablePageHandler*/
-        0 , /*ObjectFilter*/
-        0 , /*ObjectHandlerPushed*/
-        0 , /*PageCount*/
-        0 , /*ShowPageCount*/
-        1 , /*NumCopies*/
-        0 , /*NumCopiesSet*/
-        0 , /*IgnoreNumCopies*/
-        0 , /*UseCIEColor*/
-        0 , /*LockSafetyParams*/
+        }, /* ImagingBBox */
+        0, /* ImagingBBox_set */
+        { X_DPI, Y_DPI }, /* HWResolution*/
+        {
+          (float)(-(0) * (X_DPI)),
+          (float)(-(0) * (Y_DPI))
+        }, /* Margins */
+        {
+          (float)((0) * 72.0),
+          (float)((0) * 72.0),
+          (float)((0) * 72.0),
+          (float)((0) * 72.0)
+        }, /* HWMargins */
+        0,  /*FirstPage*/
+        0,  /*LastPage*/
+        0,  /*PageHandlerPushed*/
+        0,  /*DisablePageHandler*/
+        0,  /*ObjectFilter*/
+        0,  /*ObjectHandlerPushed*/
+        0,  /*PageCount*/
+        0,  /*ShowPageCount*/
+        1,  /*NumCopies*/
+        0,  /*NumCopiesSet*/
+        0,  /*IgnoreNumCopies*/
+        0,  /*UseCIEColor*/
+        0,  /*LockSafetyParams*/
         0,  /*band_offset_x*/
-        0,  /*band_offset_*/
+        0,  /*band_offset_y*/
         false, /*BLS_force_memory*/
         {false}, /*sgr*/
         0, /*MaxPatternBitmap*/
         0, /*page_uses_transparency*/
-        { MAX_BITMAP, BUFFER_SPACE,
+        {
+          MAX_BITMAP, BUFFER_SPACE,
           { BAND_PARAMS_INITIAL_VALUES },
           0/*false*/, /* params_are_read_only */
           BandingAuto /* banding_type */
         }, /*space_params*/
         0, /*icc_struct*/
         GS_UNKNOWN_TAG,         /* this device supports tags */
+        1,			/* default interpolate_control */
         {
             gx_default_install,
             gx_default_begin_page,
             gx_default_end_page
-        },
-        { 0 },
-        { 0 },
-        { bittags_print_page,
+        }, /* page_procs */
+        { 0 }, /* procs */
+        { 0 }, /* skip */
+        {
+          bittags_print_page,
           gx_default_print_page_copies,
           { bittag_create_buf_device,
             gx_default_size_buf_device,
             gx_default_setup_buf_device,
             gx_default_destroy_buf_device },
-          gx_default_get_space_params },
-        { 0 },
-        0 ,
-        0 ,
-        0 ,
-        -1,
-        0 ,
-        0,
-        0,
-        0,
-        0,
-        0,
-        0,
-        0,
-        0 ,
-        0,
-        0,
-        0
+          gx_default_get_space_params
+        }, /* printer_procs */
+        { 0 }, /* fname */
+        false, /* OpenOutputFile */
+        false, /* ReopenPerPage */
+        false, /* Duplex */
+        -1,    /* Duplex_set */
+        false, /* file_is_new */
+        NULL,  /* file */
+        0,     /* buffer_space */
+        NULL,  /* buf */
+        NULL,  /* buffer_memory */
+        NULL,  /* bandlist_memory */
+        0,     /* clist_disable_mask */
+        false, /* bg_print_requested */
+        {0},   /* bg_print */
+        0,     /* num_render_threads_requested */
+        NULL,  /* saved_pages_list */
+        {0},   /* save_procs_while_delaying_erasepage */
+        {0}    /* orig_procs */
     };
 
 static void
@@ -547,12 +557,15 @@ bittag_open(gx_device * pdev)
     return code;
 }
 
-/* fill the page fills with unmarked white */
+/*
+ * Fill the page fills with unmarked white, As with the pdf14 device, we treat
+ * GS_UNTOUCHED_TAG == 0 as an invariant
+*/
 static int
 bittag_fillpage(gx_device *dev, gs_gstate * pgs, gx_device_color *pdevc)
 {
     return (*dev_proc(dev, fill_rectangle))(dev, 0, 0, dev->width, dev->height,
-                                            GS_UNTOUCHED_TAG << 24 | 0xffffff);
+                                            0x00ffffff);	/* GS_UNTOUCHED_TAG == 0 */
 }
 
 static int
@@ -560,7 +573,6 @@ bittag_create_buf_device(gx_device **pbdev, gx_device *target, int y,
    const gx_render_plane_t *render_plane, gs_memory_t *mem,
    gx_color_usage_t *color_usage)
 {
-    gx_device_printer *ptarget = (gx_device_printer *)target;
     int code = gx_default_create_buf_device(pbdev, target, y,
         render_plane, mem, color_usage);
     set_dev_proc(*pbdev, fillpage, bittag_fillpage);
