@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -475,6 +475,9 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
   END
     fixed mx0 = 0, my0 = 0; /* See ce1_setcurrentpoint. */
 
+    /* Really this is to silence Coverity, but it makes sense and we do it a lot so no penatly */
+    memset(active_hints, 0, (max_total_stem_hints + 7) / 8);
+
     /* In case we do not get an sbw or hsbw op */
     cis.lsb.x = cis.lsb.y = cis.width.x = cis.width.y = fixed_0;
 
