@@ -661,16 +661,28 @@ again:
         return code;
     if (gs_errorname(i_ctx_p, code, &error_name) < 0)
         return code;            /* out-of-range error code! */
-    /*
-     * For greater Adobe compatibility, only the standard PostScript errors
-     * are defined in errordict; the rest are in gserrordict.
+
+    /*  If LockFilePermissions is true, we only refer to gserrordict, which
+     *  is not accessible to Postcript jobs
      */
-    if (dict_find_string(systemdict, "errordict", &perrordict) <= 0 ||
-        (dict_find(perrordict, &error_name, &epref) <= 0 &&
-         (dict_find_string(systemdict, "gserrordict", &perrordict) <= 0 ||
-          dict_find(perrordict, &error_name, &epref) <= 0))
-        )
-        return code;            /* error name not in errordict??? */
+    if (i_ctx_p->LockFilePermissions) {
+        if (((dict_find_string(systemdict, "gserrordict", &perrordict) <= 0 ||
+              dict_find(perrordict, &error_name, &epref) <= 0))
+            )
+            return code;            /* error name not in errordict??? */
+    }
+    else {
+        /*
+         * For greater Adobe compatibility, only the standard PostScript errors
+         * are defined in errordict; the rest are in gserrordict.
+         */
+        if (dict_find_string(systemdict, "errordict", &perrordict) <= 0 ||
+            (dict_find(perrordict, &error_name, &epref) <= 0 &&
+             (dict_find_string(systemdict, "gserrordict", &perrordict) <= 0 ||
+              dict_find(perrordict, &error_name, &epref) <= 0))
+            )
+            return code;            /* error name not in errordict??? */
+    }
     doref = *epref;
     epref = &doref;
     /* Push the error object on the operand stack if appropriate. */
@@ -1254,7 +1266,7 @@ remap:              if (iesp + 2 >= estop) {
             pvalue = IREF->value.pname->pvalue;
             if (!pv_valid(pvalue)) {
                 uint nidx = names_index(int_nt, IREF);
-                uint htemp;
+                uint htemp = 0;
 
                 INCR(find_name);
                 if ((pvalue = dict_find_name_by_index_inline(nidx, htemp)) == 0)
@@ -1484,9 +1496,12 @@ remap:              if (iesp + 2 >= estop) {
                         /* If the updated string isn't empty, push it back */
                         /* on the e-stack. */
                         {
-                            uint size = sbufavailable(&ss);
+                            /* This is just the available buffer size, so
+                               a signed int is plenty big
+                             */
+                            int size = sbufavailable(&ss);
 
-                            if (size) {
+                            if (size > 0) {
                                 if (iesp >= estop)
                                     return_with_error_iref(gs_error_execstackoverflow);
                                 ++iesp;
@@ -1638,7 +1653,7 @@ remap:              if (iesp + 2 >= estop) {
 
                             pvalue = name_index_ptr_inline(int_nt, nidx)->pvalue;
                             if (!pv_valid(pvalue)) {
-                                uint htemp;
+                                uint htemp = 0;
 
                                 INCR(p_find_name);
                                 if ((pvalue = dict_find_name_by_index_inline(nidx, htemp)) == 0) {
