@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -239,6 +239,10 @@ zget_device_params(i_ctx_t *i_ctx_p, bool is_hardware)
     ref *pmark;
 
     check_read_type(op[-1], t_device);
+
+    if(!r_has_type(op, t_null)) {
+        check_type(*op, t_dictionary);
+    }
     rkeys = *op;
     dev = op[-1].value.pdevice;
     pop(1);
@@ -362,13 +366,8 @@ zoutputpage(i_ctx_t *i_ctx_p)
 
         print_resource_usage(minst, &(i_ctx_p->memory), "Outputpage start");
     }
-#ifdef PSI_INCLUDED
-    code = ps_end_page_top(imemory,
-                           (int)op[-1].value.intval, op->value.boolval);
-#else
     code = gs_output_page(igs, (int)op[-1].value.intval,
                           op->value.boolval);
-#endif
     if (code < 0)
         return code;
     pop(2);
@@ -472,10 +471,6 @@ zsetdevice(i_ctx_t *i_ctx_p)
             return_error(gs_error_invalidaccess);
     }
     dev->ShowpageCount = 0;
-#ifndef PSI_INCLUDED
-    /* the language switching build shouldn't install a new device
-       here.  The language switching machinery installs a shared
-       device. */
 
     if (op->value.pdevice == 0)
         return gs_note_error(gs_error_undefined);
@@ -484,7 +479,6 @@ zsetdevice(i_ctx_t *i_ctx_p)
     if (code < 0)
         return code;
 
-#endif
     make_bool(op, code != 0);	/* erase page if 1 */
     invalidate_stack_devices(i_ctx_p);
     clear_pagedevice(istate);
